#!/usr/bin/perl
#
# $Id: migrate_passwd.pl,v 1.17 2005/03/05 03:15:55 lukeh Exp $
#
# Copyright (c) 1997-2003 Luke Howard.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#	notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#	notice, this list of conditions and the following disclaimer in the
#	documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#	must display the following acknowledgement:
#		This product includes software developed by Luke Howard.
# 4. The name of the other may not be used to endorse or promote products
#	derived from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE LUKE HOWARD ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL LUKE HOWARD BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
#
# Password migration tool. Migrates /etc/shadow as well, if it exists.
#
# Thanks to Peter Jacob Slot <peter@vision.auk.dk>.
#

require '/etc/openldap/migrate_common.ph';

use Text::Iconv;
$converter = Text::Iconv->new("UTF-8", "ASCII//TRANSLIT");

$PROGRAM = "migrate_passwd.pl";
$NAMINGCONTEXT = &getsuffix($PROGRAM);

if ($#ARGV < 0) {
	print STDERR "Usage: $PROGRAM [options] infile [outfile]\n";
	print STDERR "  Options:\n";
	print STDERR "	--minuid N	Minimal UID number to migrate\n";
	print STDERR "	--maxuid M	Maximal UID number to migrate\n";
	print STDERR "	--mingid N	Minimal GID number to migrate\n";
	print STDERR "	--maxgid M	Maximal GID number to migrate\n";
	print STDERR "	--samba		Migrate smbpasswd\n";
	print STDERR "	-s FILE		Path to shadow file (default: /etc/shadow)\n";
	print STDERR "	-S FILE		Path to smbpasswd file (default: /etc/samba/smbpasswd)\n";
	exit 1;
}

$do_samba = 0;
$shadowf = "/etc/shadow";
$smbpassf = "/etc/samba/smbpasswd";
if ( defined($IGNORE_UID_BELOW) ) {
	$minuid = $IGNORE_UID_BELOW;
}
if ( defined($IGNORE_UID_ABOVE) ) {
	$maxuid = $IGNORE_UID_ABOVE;
}
if ( defined($IGNORE_GID_BELOW) ) {
	$mingid = $IGNORE_GID_BELOW;
}
if ( defined($IGNORE_GID_ABOVE) ) {
	$maxgid = $IGNORE_GID_ABOVE;
}
while ($ARGV[0] =~ /^-.*/) {
	if ($ARGV[0] eq "--minuid") {
		$minuid = $ARGV[1];
		shift ; shift;
	} elsif ($ARGV[0] eq "--maxuid") {
		$maxuid = $ARGV[1];
		shift ; shift;
	} elsif ($ARGV[0] eq "--mingid") {
		$mingid = $ARGV[1];
		shift ; shift;
	} elsif ($ARGV[0] eq "--maxgid") {
		$maxgid = $ARGV[1];
		shift ; shift;
	} elsif ($ARGV[0] eq "--samba") {
		$do_samba = 1;
		shift;
	} elsif ($ARGV[0] eq "-s") {
		$shadowf = $ARGV[1];
		shift ; shift;
	} elsif ($ARGV[0] eq "-S") {
		$smbpassf = $ARGV[1];
		shift ; shift;
	} else {
		shift;
	}
}

if ($do_samba && !defined($DEFAULT_SMB_SID)) {
	print STDERR "You must set \$DEFAULT_SMB_SID in /etc/openldap/migrate_common.ph to migrate smbpasswd\n";
	exit 2;
}

&parse_args();
&read_shadow_file();
if ($do_samba) { &read_samba(); }
&open_files();

while(<INFILE>)
{
	chop;
	next if /^\s*$/;
	next if /^#/;
	next if /^\+/;

	local($user, $pwd, $uid, $gid, $gecos, $homedir, $shell) = split(/:/);
	next if (defined($minuid) and ($uid < $minuid));
	next if (defined($maxuid) and ($uid > $maxuid));
	next if (defined($mingid) and ($gid < $mingid));
	next if (defined($maxgid) and ($gid > $maxgid));
	
	if ($use_stdout) {
		&dump_user(STDOUT, $user, $pwd, $uid, $gid, $gecos, $homedir, $shell);
	} else {
		&dump_user(OUTFILE, $user, $pwd, $uid, $gid, $gecos, $homedir, $shell);
	}
}

sub dump_user
{
	local($HANDLE, $user, $pwd, $uid, $gid, $gecos, $homedir, $shell) = @_;
	local($name,$office,$wphone,$hphone)=split(/,/,$gecos);
	local($sn);	
	local($givenname);	
	local($cn);
	local(@tmp);

	if ($name) { $cn = $name; } else { $cn = $user; }

	$_ = $cn;
	@tmp = split(/\s+/);
	$sn = $tmp[$#tmp];
	pop(@tmp);
	$givenname=join(' ',@tmp);
	
	print $HANDLE "dn: uid=$user,$NAMINGCONTEXT\n";
	print $HANDLE "uid: $user\n";
	print $HANDLE "cn: $cn\n";

	if ($EXTENDED_SCHEMA) {
		if ($wphone) {
			print $HANDLE "telephoneNumber: $wphone\n";
		}
		if ($office) {
			print $HANDLE "roomNumber: $office\n";
		}
		if ($hphone) {
			print $HANDLE "homePhone: $hphone\n";
		}
		if ($givenname) {
			print $HANDLE "givenName: $givenname\n";
		}
		print $HANDLE "sn: $sn\n";
		if ($DEFAULT_MAIL_DOMAIN) {
			print $HANDLE "mail: $user\@$DEFAULT_MAIL_DOMAIN\n";
		}
		if ($DEFAULT_MAIL_HOST) {
			print $HANDLE "mailRoutingAddress: $user\@$DEFAULT_MAIL_HOST\n";
			print $HANDLE "mailHost: $DEFAULT_MAIL_HOST\n";
			print $HANDLE "objectClass: inetLocalMailRecipient\n";
		}
		print $HANDLE "objectClass: person\n";
		print $HANDLE "objectClass: organizationalPerson\n";
		print $HANDLE "objectClass: inetOrgPerson\n";
	} else {
		print $HANDLE "objectClass: account\n";
	}

	print $HANDLE "objectClass: posixAccount\n";
	print $HANDLE "objectClass: top\n";

	if ($DEFAULT_REALM) {
		print $HANDLE "objectClass: krb5Principal\n";
		print $HANDLE "objectClass: krb5KDCEntry\n";
		print $HANDLE "krb5PrincipalName: $user\@$DEFAULT_REALM\n";
		print $HANDLE "krb5KeyVersionNumber: 0\n";
		print $HANDLE "krb5KDCFlags: 126\n";
		print $HANDLE "krb5MaxRenew: 604800\n";
		print $HANDLE "krb5MaxLife: 86400\n";
	}

	if ($DEFAULT_SMB_SID) {
		my $userSID = (2 * $uid) + 1000;
		my $groupSID = (2 * $gid) + 1001;
		print $HANDLE "objectClass: sambaSamAccount\n";
		print $HANDLE "displayName: $cn\n";
		print $HANDLE "sambaSID: $DEFAULT_SMB_SID-$userSID\n";
		print $HANDLE "sambaPrimaryGroupSID: $DEFAULT_SMB_SID-$groupSID\n";
		if ($do_samba) {
			print $HANDLE "sambaLMPassword: ".$sambaUsers{$user}->{"sambaLMPassword"}."\n";
			print $HANDLE "sambaNTPassword: ".$sambaUsers{$user}->{"sambaNTPassword"}."\n";
			print $HANDLE "sambaAcctFlags: ".$sambaUsers{$user}->{"sambaAcctFlags"}."\n";
			print $HANDLE "sambaPwdLastSet: ".$sambaUsers{$user}->{"sambaPwdLastSet"}."\n";
		}
	}

	if ($shadowUsers{$user} ne "") {
		&dump_shadow_attributes($HANDLE, split(/:/, $shadowUsers{$user}));
	} else {
		print $HANDLE "userPassword: {crypt}$pwd\n";
	}

	if ($shell) {
		print $HANDLE "loginShell: $shell\n";
	}

	if ($uid ne "") {
		print $HANDLE "uidNumber: $uid\n";
	} else {
		print $HANDLE "uidNumber:\n";
	}

	if ($gid ne "") {
		print $HANDLE "gidNumber: $gid\n";
	} else {
		print $HANDLE "gidNumber:\n";
	}

	if ($homedir) {
		print $HANDLE "homeDirectory: $homedir\n";
	} else {
		print $HANDLE "homeDirectory:\n";
	}

	if ($gecos) {
		$cgecos = $converter->convert($gecos);
		print $HANDLE "gecos: $cgecos\n";
	}

	print $HANDLE "\n";
}

close(INFILE);
if (OUTFILE != STDOUT) { close(OUTFILE); }

sub read_shadow_file
{
	open(SHADOW, $shadowf) || return;
	while(<SHADOW>) {
		chop;
		($shadowUser) = split(/:/, $_);
		$shadowUsers{$shadowUser} = $_;
	}
	close(SHADOW);
}

sub dump_shadow_attributes
{
	local($HANDLE, $user, $pwd, $lastchg, $min, $max, $warn, $inactive, $expire, $flag) = @_;

	print $HANDLE "objectClass: shadowAccount\n";
	if ($pwd) {
		print $HANDLE "userPassword: {crypt}$pwd\n";
	}
	if ($lastchg ne "") {
		print $HANDLE "shadowLastChange: $lastchg\n";
	}
	if ($min) {
		print $HANDLE "shadowMin: $min\n";
	}
	if ($max) {
		print $HANDLE "shadowMax: $max\n";
	}
	if ($warn) {
		print $HANDLE "shadowWarning: $warn\n";
	}
	if ($inactive) {
		print $HANDLE "shadowInactive: $inactive\n";
	}
	if ($expire) {
		print $HANDLE "shadowExpire: $expire\n";
	}
	if ($flag) {
		print $HANDLE "shadowFlag: $flag\n";
	}
}

sub read_samba
{
	open(INPUT, "<$smbpassf");
	while (<INPUT>) {
		my ($sambaUser, $id, $lmp, $ntp, $f, $lf, $xxx) = split(':');
		$sambaUsers{$sambaUser}->{"sambaLMPassword"} = $lmp;
		$sambaUsers{$sambaUser}->{"sambaNTPassword"} = $ntp;
		$sambaUsers{$sambaUser}->{"sambaAcctFlags"} = $f;
		$lf =~ s/^LCT-//;
		$sambaUsers{$sambaUser}->{"sambaPwdLastSet"} = hex($lf);
	}
	close(INPUT);
}
